 /* IOLib-Filters-C, Filter package for IOLib-C
  * Copyright (C) 1998-2001 Robert Kausch <robert.kausch@gmx.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free
  * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  * MA 02111-1307, USA */

#ifndef __IOLIB_FILTERS_BZ2_
#define __IOLIB_FILTERS_BZ2_

#include <iolib-c.h>
#include <filter-bz2-c.h>
#include <bzlib.h>
#include <memory.h>
#include <stdlib.h>

void FilterBZ2_Init_real(FilterBZ2 *filter)
{
	filter->filter = malloc(sizeof(IOLibFilter));

	IOLibFilter_New(filter->filter);

	filter->filter->packsize = 4;

	IOLibFilter_SetEncodeProc(filter->filter, FilterBZ2_EncodeProc, (void *) filter);
	IOLibFilter_SetDecodeProc(filter->filter, FilterBZ2_DecodeProc, (void *) filter);
}

void FilterBZ2_Free_real(FilterBZ2 *filter)
{
	IOLibFilter_Free(filter->filter);

	free(filter->filter);
}

void FilterBZ2_EncodeProc(IOLibFilter *dummy, unsigned char **data, int size, int *outsize, void *filter)
{
	unsigned char	*dest;

	*outsize = size + size / 100 + 1000;

	dest = malloc(*outsize);

	BZ2_bzBuffToBuffCompress((char *) dest, (unsigned int *) outsize, (char *) *data, size, 5, 0, 0);

	free(*data);

	*data = malloc(*outsize);

	memcpy((void *) *data, (void *) dest, *outsize);

	free(dest);
}

void FilterBZ2_DecodeProc(IOLibFilter *dummy, unsigned char **data, int size, int *outsize, void *filter)
{
	unsigned char	*dest;

	// try to guess the size of the uncompressed data

	if (size < (200 * 1024))		*outsize = size * 100 + 10000;
	else if (size > (20 * 1024 * 1024))	*outsize = size * 5 + 10000;
	else					*outsize = size * 10 + 10000;

	dest = malloc(*outsize);

	BZ2_bzBuffToBuffDecompress((char *) dest, (unsigned int *) outsize, (char *) *data, size, 0, 0);

	free(*data);

	*data = malloc(*outsize);

	memcpy((void *) *data, (void *) dest, *outsize);

	free(dest);
}

IOLibFilter *FilterBZ2_GetFilter(FilterBZ2 *filter)
{
	return filter->filter;
}

#endif
