 /* IOLib-Filters-C, Filter package for IOLib-C
  * Copyright (C) 1998-2001 Robert Kausch <robert.kausch@gmx.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free
  * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  * MA 02111-1307, USA */

#ifndef __IOLIB_FILTERS_XOR_
#define __IOLIB_FILTERS_XOR_

#include <iolib-c.h>
#include <filter-xor-c.h>

void FilterXOR_Init_real(FilterXOR *filter)
{
	filter->filter = malloc(sizeof(IOLibFilter));

	IOLibFilter_New(filter->filter);

	filter->filter->packsize = 4;

	IOLibFilter_SetEncodeProc(filter->filter, FilterXOR_EncodeProc, (void *) filter);
	IOLibFilter_SetDecodeProc(filter->filter, FilterXOR_DecodeProc, (void *) filter);

	filter->modifier = 0;
}

void FilterXOR_Free_real(FilterXOR *filter)
{
	IOLibFilter_Free(filter->filter);

	free(filter->filter);
}

void FilterXOR_EncodeProc(IOLibFilter *dummy, unsigned char **data, int size, int *outsize, void *filter)
{
	unsigned int value;

	*outsize = size;

	value = (*data)[3] + 256 * (*data)[2] + 65536 * (*data)[1] + 16777216 * (*data)[0];

	value ^= ((FilterXOR *) filter)->modifier;

	(*data)[3] = value % 256;
	(*data)[2] = value % 16777216 % 65536 / 256;
	(*data)[1] = value % 16777216 / 65536;
	(*data)[0] = value / 16777216;
}

void FilterXOR_DecodeProc(IOLibFilter *dummy, unsigned char **data, int size, int *outsize, void *filter)
{
	FilterXOR_EncodeProc(dummy, data, size, outsize, filter);
}

void FilterXOR_SetModifier(FilterXOR *filter, int mod)
{
	filter->modifier = mod;
}

IOLibFilter *FilterXOR_GetFilter(FilterXOR *filter)
{
	return filter->filter;
}

#endif
